"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var http_1 = __importDefault(require("http"));
var https_1 = __importDefault(require("https"));
var express = require("express");
var router = express.Router();
var app = express();
var auth = require("express-openid-connect").auth;
var favicon = require("serve-favicon");
var fs = require("fs");
var path = require("path");
var url = require("url");
var session = require("express-session");
var stringReplaceStream = require("string-replace-stream");
var bodyParser = require("body-parser");
var multipart = require("connect-multiparty");
var multipartMiddleware = multipart();
var options = require("nghost/lib/options");
module.exports = function (option) {
    if (option === void 0) { option = options; }
    options = mergeDeep(options, option);
    if (!fs.existsSync(options.publicFolder)) {
        console.error("ERROR: Public folder not found.");
        throw "ERROR: Public folder not found.";
    }
    if (fs.existsSync(options.favicon))
        app.use(favicon(options.favicon));
    var sess = {
        secret: options.session || options.auth0.AUTH0_CLIENT_SECRET,
        cookie: { secure: false },
        resave: false,
        saveUninitialized: true,
    };
    if (options.bodyParser) {
        app.use(bodyParser.json());
        app.use(bodyParser.urlencoded({
            extended: true,
        }));
    }
    app.use(function (req, res, next) {
        res.header("Access-Control-Allow-Origin", options["Access-Control-Allow-Origin"]);
        res.header("Access-Control-Allow-Headers", options["Access-Control-Allow-Headers"]);
        next();
    });
    ////#region SimpleAuth
    if (options.simpleAuth.enable) {
        app.use(function (req, res, next) {
            var clientIp = req.headers["x-forwarded-for"] || req.connection.remoteAddress;
            var allowed = false;
            options.simpleAuth.allowip.map(function (ai) {
                var replaced = ai.replace("*", "[0-9]{1}");
                var regex = new RegExp(replaced, "g");
                if (!allowed) {
                    if (clientIp.match(regex))
                        allowed = true;
                }
            });
            // -----------------------------------------------------------------------
            // parse login and password from headers
            var b64auth = (req.headers.authorization || "").split(" ")[1] || "";
            var _a = Buffer.from(b64auth, "base64")
                .toString()
                .split(":"), login = _a[0], password = _a[1];
            // Verify login and password are set and correct
            if ((login &&
                password &&
                login === option.simpleAuth.username &&
                password === option.simpleAuth.password) ||
                allowed) {
                // Access granted...
                return next();
            }
            // Access denied...
            res.set("WWW-Authenticate", 'Basic realm="401"'); // change this
            res.status(401).send("Authentication required."); // custom message
            // -----------------------------------------------------------------------
        });
    }
    ////#endregion SimpleAuth
    //#region Auth0
    if (options.session)
        sess.cookie.secure = true;
    if (options.auth0.enable) {
        app.use(session(sess));
        var config = {
            authRequired: false,
            auth0Logout: true,
            secret: options.session,
            issuerBaseURL: options.auth0.AUTH0_DOMAIN,
            baseURL: options.auth0.AUTH0_CALLBACK_URL,
            clientID: options.auth0.AUTH0_CLIENT_ID,
            clientSecret: options.auth0.AUTH0_CLIENT_SECRET,
        };
        app.use(auth(config));
        app.use(function (req, res, next) {
            res.locals.user = req.oidc.user;
            res.locals.isAuthenticated = req.oidc.isAuthenticated();
            next();
        });
        app.use("/", router);
    }
    //#endregion Auth0
    app.get(/.*/, multipartMiddleware, function (req, res) {
        FnRequest("gets", req, res);
    });
    app.post(/.*/, multipartMiddleware, function (req, res) {
        FnRequest("posts", req, res);
    });
    app.put(/.*/, multipartMiddleware, function (req, res) {
        FnRequest("puts", req, res);
    });
    app.delete(/.*/, multipartMiddleware, function (req, res) {
        FnRequest("deletes", req, res);
    });
    if (!options.https) {
        var httpServer = http_1.default.createServer(app);
        httpServer.listen(options.port);
        // app.listen(options.port);
    }
    if (options.https) {
        var credentials = {};
        try {
            credentials.key = fs.readFileSync(options.key);
            credentials.cert = fs.readFileSync(options.cert);
            credentials.ca = fs.readFileSync(options.ca);
        }
        catch (err) { }
        var httpsServer = https_1.default.createServer(credentials, app);
        httpsServer.listen(options.port);
    }
    return app;
};
var FnRequest = function (type, req, res) {
    var exist = false;
    if (req.url.substring(0).search(/[^.]*$/) <= 0) {
        var requests = options[type].sort(function (a, b) {
            return a.key.toString().length < b.key.toString().length ? 1 : -1;
        });
        requests.forEach(function (get) {
            try {
                if (req.url.match(get.key)) {
                    exist = true;
                    get.function(req, res);
                    throw {};
                }
            }
            catch (Exept) {
                //   if (Exept) console.error("Error :", Exept);
            }
        });
    }
    if (!exist) {
        getGenericRequest(req, res);
    }
};
var getGenericRequest = function (req, res) {
    var pathName = url
        .parse(req.url)
        .pathname.split("/")
        .filter(function (e) {
        return e.length > 0;
    });
    if (pathName.length == 0) {
        res.setHeader("Content-Type", "text/html");
        res.write(fs.readFileSync(path.join(options.publicFolder, "index.html"), "utf-8"));
        res.end();
    }
    else
        checkReqAndSend(pathName.join("/"), res);
};
var checkReqAndSend = function (req, res) {
    var exist = false;
    for (var ctn in options.contents) {
        if (req.match(options.contents[ctn].key)) {
            exist = true;
            if (options.contents[ctn].type == "text") {
                responseText(res, req, options.contents[ctn].contentType);
            }
            else {
                responseBinary(res, req, options.contents[ctn].contentType);
            }
            break;
        }
    }
    if (!exist) {
        if (req.match(/^\/[a-zA-Z]{1}[a-zA-Z0-9]{2,8}$/)) {
            var file = path.join(options.publicFolder, req) + ".html";
            if (fs.existsSync(file)) {
                responseText(res, "".concat(req, ".html"), "text/html");
            }
            else {
                response(res, "404", "<h1 style=\"color:green; font-size:5em; font-weight:100;\">\n                Ok Successful route But We Don't find the file...\n            </h1>");
            }
        }
        else {
            notFound(req, res, options.publicFolder);
        }
    }
};
var responseText = function (res, file, contentType) {
    response(res, file, contentType, options.encode);
};
var responseBinary = function (res, file, contentType) {
    response(res, file, contentType);
};
var response = function (res, file, contentType, encode) {
    if (encode === void 0) { encode = null; }
    if (file != "404") {
        res.writeHead(200, {
            "Content-type": contentType,
        });
        if (encode) {
            var stream = fs
                .createReadStream(path.join(options.publicFolder, file), encode)
                .pipe(stringReplaceStream("localhost:" + options.port, options.hostname + ":" + options.port));
        }
        else {
            var stream = fs.createReadStream(path.join(options.publicFolder, file));
        }
        stream.pipe(res);
    }
    else {
        res.writeHead(404, {
            "Content-type": "text/html",
        });
        res.end("\n              ".concat(contentType, "\n          "));
    }
};
/**
 * Simple object check.
 * @param item
 * @returns {boolean}
 */
var isObject = function (item) {
    return item && typeof item === "object" && !Array.isArray(item);
};
/**
 * Deep merge two objects.
 * @param target
 * @param ...sources
 */
var mergeDeep = function (target) {
    var _a, _b;
    var sources = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        sources[_i - 1] = arguments[_i];
    }
    if (!sources.length)
        return target;
    var source = sources.shift();
    if (isObject(target) && isObject(source)) {
        for (var key in source) {
            if (isObject(source[key])) {
                if (!target[key])
                    Object.assign(target, (_a = {},
                        _a[key] = {},
                        _a));
                mergeDeep(target[key], source[key]);
            }
            else {
                Object.assign(target, (_b = {},
                    _b[key] = source[key],
                    _b));
            }
        }
    }
    return mergeDeep.apply(void 0, __spreadArray([target], sources, false));
};
var notFound = function (req, res, publicfolder) {
    res.sendFile("index.html", {
        root: publicfolder,
    });
};
